using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.TeamFoundation.Client;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
using Microsoft.TeamFoundation.VersionControl.Client;
using Microsoft.TeamFoundation.WorkItemTracking.Controls;

namespace CegTFSHelpers
{
	/// <summary>
	/// Form for a Work Item selection
	/// </summary>
	public partial class WISelectionForm : Form
	{
//		MainForm				m_Owner	= null;
		TeamFoundationServer	m_TFS	= null;
		WorkItemStore			m_WIS	= null;
		Project					m_WIP	= null;
		WorkItem				_WI		= null;

		/// <summary>
		/// Get the Selected Work Item's ID
		/// </summary>
		public int				m_SelectedWorkItemID
		{
			get
			{
				return (_WI==null) ? 0 : _WI.Id;
			}
		}

		WorkItem m_WI
		{
			get
			{
				return _WI;
			}

			set
			{
				_WI = value;
				if (m_OK != null)
				{
					m_OK.Enabled = value != null;
					m_OK.Update();
				}
				if (m_WIForm != null)
				{
					try
					{
						m_WIForm.Item = value;
					}
					catch (Exception ex)
					{
						ex.ToString();
					}
				}
			}

		}
		StoredQuery m_Query
		{
			get
			{
				// Checking
				if (m_WIP == null)
				{
					return null;
				}

				String qname = (String)m_QueryList.SelectedItem;
				foreach (StoredQuery sq in m_WIP.StoredQueries)
				{
					if (sq.Name == qname)
					{
						return m_WIP.StoredQueries[sq.QueryGuid];
					}
				}
				return null;
			}
		}

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="tfs">Used Team Foundation Server</param>
		/// <param name="wis">Used Work Item Store</param>
		public WISelectionForm(TeamFoundationServer tfs, WorkItemStore wis)
		{
//			m_Owner = owner;
			m_TFS	= tfs;
			m_WIS	= wis;

			InitializeComponent();
			m_QueryGo.Enabled = false;
			m_WIDGo.Enabled = false;

			// On cre la collection de champs que l'on souhaite afficher
			DisplayFieldCollection Fields = new DisplayFieldCollection();
			Fields.Clear();
			Fields.Add(new DisplayField("ID", 50));
			Fields.Add(new DisplayField("Work Item Type", 200));
			Fields.Add(new DisplayField("Assigned To", 200));
			Fields.Add(new DisplayField("Title", 200));

			// On cre les colonnes  afficher
			m_WIResult.ResultOptions = new ResultOptions(Fields, new SortFieldCollection());
			m_WIResult.SelectionChanged += new EventHandler(WIResult_SelectionChanged);
		}

		/// <summary>
		/// Set the TeamProject to use for the selection
		/// </summary>
		/// <param name="tp">The Team Project to use</param>
		public void SetTeamProject(Project tp)
		{
			m_WIP= tp;

			//RaZ du Work Item slectionn
			m_WI = null;

			UpdateQueries();
		}

		private void UpdateQueries()
		{
			m_QueryList.Items.Clear();

			if (m_WIP != null)
			{
				StoredQueryCollection sqc = m_WIP.StoredQueries;
				foreach (StoredQuery sq in sqc)
				{
					m_QueryList.Items.Add(sq.Name);
				}
			}
		}

		private void WIResult_SelectionChanged(object sender, EventArgs e)
		{
			// Il faut un lment slectionn
			if (m_WIResult.SelectedRows.Count == 0)
			{
				return;
			}
			m_WI = m_WIResult.WorkItemCollection[m_WIResult.SelectedRows[0].Index];
		}

		private void QuerySelectionChanged(object sender, EventArgs e)
		{
			m_QueryGo.Enabled = m_Query.QueryText != "";
			m_WIForm.Item = null;
			RefreshQuery(sender, e);
		}

		private void RefreshQuery(object sender, EventArgs e)
		{
			//// On charge la grille avec le rsultat de la requte
			//using (new MainForm.Action(m_Owner, @"Performing the query...", @"Query completed successfully"))
			//{
				// Set the cursor to wait
				Cursor.Current = Cursors.WaitCursor;

				// We associate the @projet variable to the current Team Project
				Hashtable h = new Hashtable();
				h.Add("project", m_WIP.Name);

				m_WIResult.WorkItemCollection = m_WIS.Query(m_Query.QueryText, h);

				Cursor.Current = Cursors.Default;					// Release the Wait Cursor by putting the default one
//			}
		}

		private void WIIDChanged(object sender, EventArgs e)
		{
			try
			{
				int id = int.Parse(m_WIID.Text);
				WorkItem wi = m_WIS.GetWorkItem(id);

				if ((wi!=null) && (wi.Project == m_WIP))
				{
					m_WIDGo.Enabled = true;
					m_WIDGo.Update();
				}
			} catch (Exception)
			{
				m_WIDGo.Enabled = false;
				m_WIDGo.Update();
			}
		}

		private void RefreshWIID(object sender, EventArgs e)
		{
			try
			{
				int id = int.Parse(m_WIID.Text);
				m_WIResult.WorkItemCollection = m_WIS.Query(String.Format(@"SELECT System.ID from WorkItems where [System.ID]={0}", id));
			}
			catch (Exception)
			{
				m_WIDGo.Enabled = false;
				m_WIDGo.Update();
			}
		}

		private void SelIDKeyPress(object sender, KeyPressEventArgs e)
		{
			if ((m_WIDGo.Enabled==true) && (e.KeyChar=='\r'))
			{
				RefreshWIID(sender, e);
			}
		}

		private void m_WIResult_CellMouseDoubleClick(object sender, DataGridViewCellMouseEventArgs e)
		{
			DialogResult = DialogResult.OK;
			Close();
		}
	}
}