using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using CegWIManager;
using Microsoft.TeamFoundation.Server;
using Microsoft.TeamFoundation.Client;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
using Microsoft.TeamFoundation.VersionControl.Client;
using Microsoft.TeamFoundation.WorkItemTracking.Controls;
using Aga.Controls.Tree;
using Aga.Controls.Tree.NodeControls;
using System.Reflection;

namespace CegWICreator
{
	public partial class BranchesComparisonForm : Form
	{
		class BranchInfo
		{
			public String m_Path;
			public Changeset[] m_ChangesetList;
			public int m_ColumnID;

			public BranchInfo(String path, int columnid)
			{
				m_Path = path;
				m_ColumnID = columnid;
			}
		}

		class WINode : Aga.Controls.Tree.Node
		{
			public string WIBranch0 = "No", WIBranch1 = "No", WIBranch2 = "No", WIBranch3 = "No", WIBranch4 = "No";
			public string WIBranch5 = "No", WIBranch6 = "No", WIBranch7 = "No", WIBranch8 = "No", WIBranch9 = "No";
			public string WIBranch10 = "No", WIBranch11 = "No", WIBranch12 = "No", WIBranch13 = "No", WIBranch14 = "No";
			public string WIBranch15 = "No", WIBranch16 = "No", WIBranch17 = "No", WIBranch18 = "No", WIBranch19 = "No";

			public String[] WIBranches = new String[20];

			public WINode()
			{
				for (int i=0; i<20; i++)
				{
					WIBranches[i] = "No";
				}
			}

			public string GetWIBranch(int index)
			{
				Type t = typeof(WINode);
				FieldInfo fi = t.GetField(string.Format("WIBranch{0}", index), BindingFlags.Instance | BindingFlags.Public | BindingFlags.NonPublic);
				return (string)fi.GetValue(this);
			}

			public void SetWIBranch(int index, string value)
			{
				Type t = typeof(WINode);
				FieldInfo fi = t.GetField(string.Format("WIBranch{0}", index), BindingFlags.Instance | BindingFlags.Public | BindingFlags.NonPublic);
				fi.SetValue(this, value);
			}

		}

		MainForm m_Owner = null;
		TeamFoundationServer m_TFS = null;
		WorkItemStore m_WIS = null;
		VersionControlServer m_SC = null;
		WINetworkModel m_NetModel = null;

		DateTime m_From, m_To;

		List<BranchInfo> m_BranchesList;

		BranchInfo m_CurBranch;

		TreeModel m_TreeModel;
		ImageList m_IL;


		public BranchesComparisonForm(MainForm owner, TeamFoundationServer tfs, ListView.ListViewItemCollection items, DateTime from, DateTime to)
		{
			InitializeComponent();

			m_Owner = owner;
			m_TFS = tfs;
			m_SC = m_TFS.GetService(typeof(VersionControlServer)) as VersionControlServer;
			m_WIS = m_TFS.GetService(typeof(WorkItemStore)) as WorkItemStore;
			m_NetModel = new WINetworkModel(m_WIS, null);

			m_From = from;
			m_To = to;

			m_TreeModel = new TreeModel();
			WITreeView.Model = m_TreeModel;

			m_IL = m_Owner.TreeIL;

			m_BranchesList = new List<BranchInfo>();

			int columnid = 1;
			foreach(ListViewItem lvi in items)
			{
				if (lvi.Checked == true)
				{
					TreeColumn tc = new TreeColumn(lvi.Text.Substring(lvi.Text.LastIndexOf('/') + 1), 120);
					WITreeView.Columns.Add(tc);

					NodeComboBox ncc = new NodeComboBox();
					ncc.DropDownItems.AddRange(new string[] { "Yes", "No", "Partially" });
					ncc.EditEnabled = true;
					ncc.Parent = WITreeView;
					ncc.ParentColumn = tc;
					ncc.UseCompatibleTextRendering = true;
					ncc.DataPropertyName = string.Format("WIBranch{0}", columnid-1);

					WITreeView.NodeControls.Add(ncc);

					m_BranchesList.Add(new BranchInfo(lvi.Text, columnid++));
				}
			}
		}

		private void BranchesComparisonForm_Load(object sender, EventArgs e)
		{
		}

		private void BranchesComparisonForm_Shown(object sender, EventArgs e)
		{
			foreach (BranchInfo curbranch in m_BranchesList)
			{
				StatusText.Text = String.Format("Getting history of branch {0}...", curbranch.m_Path);
				Application.DoEvents();
				System.Collections.IEnumerable changesets = m_SC.QueryHistory(curbranch.m_Path, VersionSpec.Latest, 0, RecursionType.Full, null, new DateVersionSpec(m_From), new DateVersionSpec(m_To), int.MaxValue, true, false, false);

				List<Changeset> cslist = new List<Changeset>();
				foreach (Changeset curcs in changesets)
				{
					cslist.Add(curcs);
				}

				Application.DoEvents();

				curbranch.m_ChangesetList = cslist.ToArray();
				StatusText.Text = "Done.";
			}

			foreach (BranchInfo curbranch in m_BranchesList)
			{
				if (curbranch.m_ChangesetList.Length == 0)
					continue;

				m_CurBranch = curbranch;

				StatusText.Text = string.Format("Getting Work Items of branch {0}...", m_CurBranch.m_Path);

				StatusProgress.Minimum = 1;
				StatusProgress.Maximum = m_CurBranch.m_ChangesetList.Length;
				StatusProgress.Value = 1;
				foreach (Changeset curcs in m_CurBranch.m_ChangesetList)
				{
					foreach (WorkItem wi in curcs.WorkItems)
					{
						AddWIToTree(wi.Id);
					}

					StatusProgress.Increment(1);

					Application.DoEvents();
				}

				StatusProgress.Value = 1;
				StatusText.Text = "Done";
			}

			StatusText.Text = "Update Branch State...";
			foreach (WINode curnode in m_TreeModel.Nodes)
			{
				UpdateNodeBranchStatus(curnode);
			}
			WITreeView.Update();
			StatusText.Text = "Branch State updated!";
		}

		private void UpdateNodeBranchStatus(WINode node)
		{
			// Process children first, as we want to have their state correct before processing this one
			foreach (WINode curnode in node.Nodes)
			{
				UpdateNodeBranchStatus(curnode);
			}

			// Process each branch
			foreach(BranchInfo curbranch in m_BranchesList)
			{
				int curbranchi = curbranch.m_ColumnID-1;

				// Update the status of this node, depending on the direct children
				string nodestate = node.GetWIBranch(curbranchi);
				foreach (WINode curnode in node.Nodes)
				{
					if (nodestate != curnode.GetWIBranch(curbranchi))
					{
						node.SetWIBranch(curbranchi, "Partially");
						break;
					}
				}
			}
		}

		private void AddWIToTree(int wiid)
		{
			// Get the Work Item
			WorkItem wi = m_NetModel.GetWorkItem(wiid);

			// Nothing ???
			if (wi == null)
			{
				return;
			}

			// Start from the given Work Item, find the Root, and store the whole path between these two
			List<WorkItem> wilist = new List<WorkItem>();

			WorkItem curwi = wi;
			while (curwi != null)
			{
				wilist.Insert(0, curwi);				// As we go up in the hierarchy, we insert always at the first position
				curwi = m_NetModel.WIGetParent(curwi);	// to keep the right direction after
			}

			WINode tn = null;
			Collection<Aga.Controls.Tree.Node> tnc = m_TreeModel.Nodes;

			int i = m_CurBranch.m_ColumnID - 1;
			foreach (WorkItem _curwi in wilist)
			{
				tn = GetTreeNode(tnc, _curwi, true);
				tn.SetWIBranch(i, "Yes");
				tn.WIBranches[i] = "Yes";

				tnc = tn.Nodes;
			}
		}

		private WINode GetTreeNode(Collection<Aga.Controls.Tree.Node> owner, WorkItem wi, bool create)
		{
			foreach (Aga.Controls.Tree.Node curtn in owner)
			{
				WorkItem curwi = curtn.Tag as WorkItem;
				if (curwi == null)
				{
					continue;
				}

				if (curwi.Id == wi.Id)
				{
					return curtn as WINode;
				}
			}

			if (create == false)
			{
				return null;
			}

			return CreateTreeNode(wi, owner);
		}

		private WINode CreateTreeNode(WorkItem wi, Collection<Aga.Controls.Tree.Node> owner)
		{
			String name = String.Format("[{0}] {1}-{2}", wi.Id.ToString(), wi.Title, wi.State);

			// Create the node
			WINode tn = new WINode();
			tn.Text = name;
			owner.Add(tn);
			tn.Tag = wi;

			// Picture index from the Work Item's type
			int ii = m_IL.Images.IndexOfKey(wi.Type.Name);
			if (ii == -1)
			{
				ii = m_IL.Images.IndexOfKey("Default");
			}

			tn.Image = m_IL.Images[ii];

			return tn;
		}

		private void WITreeView_Collapsing(object sender, TreeViewAdvEventArgs e)
		{
			WINode node = e.Node.Tag as WINode;
			if (node == null)
			{
				return;
			}

			// Process each branch
			foreach (BranchInfo curbranch in m_BranchesList)
			{
				int curbranchi = curbranch.m_ColumnID - 1;

				// Update the status of this node, depending on the direct children
				string nodestate = node.GetWIBranch(curbranchi);
				foreach (WINode curnode in node.Nodes)
				{
					if (nodestate != curnode.GetWIBranch(curbranchi))
					{
						node.SetWIBranch(curbranchi, "Partially");
						break;
					}
				}
			}
		}

		private void WITreeView_Expanding(object sender, TreeViewAdvEventArgs e)
		{
			WINode node = e.Node.Tag as WINode;
			if (node == null)
			{
				return;
			}

			// Process each branch
			foreach (BranchInfo curbranch in m_BranchesList)
			{
				int curbranchi = curbranch.m_ColumnID - 1;

				node.SetWIBranch(curbranchi, node.WIBranches[curbranchi]);
			}

		}
	}
}