using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.Net;
using System.IO;
using System.Security.Cryptography.X509Certificates;
using CegWIManager.ProjectServer;

namespace CegWIManager
{
	public class PSManager
	{
		private	String	m_Cookie = "";
		private	String	m_URL;
		private	PDS		m_PDS = null;

		public bool Connect(String url)
		{
			// Dj connect ???
            if(m_Cookie != "") return true;

			// On stocke l'url
            m_URL = url;
			
            HttpWebRequest webreq = (HttpWebRequest)WebRequest.Create(m_URL + "/LgnIntAu.asp");
            CookieContainer cookiectn = new CookieContainer();
            webreq.CookieContainer = cookiectn;
            webreq.Credentials = CredentialCache.DefaultCredentials;
			
            //This line of code is necessary in order to work with SSL enabled
            // Project Server sites - see the SSL Certificate Handling Class below.
            ServicePointManager.CertificatePolicy = new MyCertificateValidation();

            // Return the response. 
            try
            {
                HttpWebResponse webres = (HttpWebResponse)webreq.GetResponse();
                XmlDocument doc = new XmlDocument();
                doc.Load(webres.GetResponseStream());

                // Close the response to free resources.
                webres.Close();
                m_Cookie = GetLogonStatus(doc);
                if(m_Cookie.Length < 10)
                {
                    m_Cookie = "";
					throw new Exception("PSM0001: Microsoft Project Server logon failed.\nLogon Status = " + m_Cookie);
                }
            }
            catch (WebException ex)
            {
                string mess;
                if(ex.Response == null)
                {
                    mess = "PSM0002: Logon Exception Occurred\n" + ex.Message;
                }
                else
                {
                    mess = "PSM0002: Logon Exception Occurred\n" + ex.Message + "\n" + ex.Response.ResponseUri.AbsoluteUri;
                }
				throw new Exception(mess);
            }

			// On cre le PDS
			m_PDS = new PDS();
            m_PDS.Credentials = CredentialCache.DefaultCredentials;

			Reply rep = ExecPDSInfo();
			String pipo = rep.UserName;

            return true;
		}

		private String ExecCmd(String xmltext)
		{
            XmlDocument XmlDoc = new XmlDocument();
            XmlDoc.LoadXml(xmltext);
            return m_PDS.SoapXMLRequest(m_Cookie, XmlDoc.InnerXml);
		}

		public Reply ExecPDSInfo()
		{
			String status = ExecCmd(@"<Request><PDSInfo></PDSInfo></Request>");

			XmlSerializer	xs = new XmlSerializer(typeof(Reply));
			StringReader	sr = new StringReader(status);
			return xs.Deserialize(sr) as Reply;
		}

		public void ExecProjectsStatus()
		{
			String xml =  @"<Request>
								<ProjectsStatus>
									<ProjectManager>BAUMANN LOIC</ProjectManager>
								</ProjectsStatus>
							</Request>";

			String status = ExecCmd(xml);

			String pipo = status;
		}

		public void ExecAssignmentsGet()
		{
			String xml = @"<Request>
							<AssignmentsGet>
								<Resources>
									<Resource>
										<ResourceName>BAUMANN LOIC</ResourceName>
									</Resource>
								</Resources>
								<AllAssignments>1</AllAssignments>
							</AssignmentsGet>
						   </Request>";

			String status = ExecCmd(xml);

			String pipo = status;
		}

        private string GetLogonStatus(XmlDocument XMLDoc)
        {	
            XmlNode Root;
            string Result;

            Result = "";
            Root = XMLDoc.DocumentElement;
            if(Root.LastChild.Name == "Cookie")
            {
                Result = Root.LastChild.InnerText;
            }
            else
            {
                Result = Root.FirstChild.InnerText;
            }
						
            return(Result);		
        }

	}

    //The following class is needed for handling SSL certificates.
    //This class will check the validity of a certificate and pop
    //up a message to allow the user to choose whether to continue
    //or not in the case of a potentially invalid certificate
	#region SSL Certificate Handling Class
    public class MyCertificateValidation : ICertificatePolicy
    {
        // Default policy for certificate validation.
        public static bool CheckDefaultValidate;

        public  enum    CertificateProblem  : long
        {
            CertEXPIRED                   = 0x800B0101,
            CertVALIDITYPERIODNESTING     = 0x800B0102,
            CertROLE                      = 0x800B0103,
            CertPATHLENCONST              = 0x800B0104,
            CertCRITICAL                  = 0x800B0105,
            CertPURPOSE                   = 0x800B0106,
            CertISSUERCHAINING            = 0x800B0107,
            CertMALFORMED                 = 0x800B0108,
            CertUNTRUSTEDROOT             = 0x800B0109,
            CertCHAINING                  = 0x800B010A,
            CertREVOKED                   = 0x800B010C,
            CertUNTRUSTEDTESTROOT         = 0x800B010D,
            CertREVOCATION_FAILURE        = 0x800B010E,
            CertCN_NO_MATCH               = 0x800B010F,
            CertWRONG_USAGE               = 0x800B0110,
            CertUNTRUSTEDCA               = 0x800B0112
        }

        public bool CheckValidationResult(ServicePoint sp, X509Certificate cert,
            WebRequest request, int problem)
        {   
            if (problem == 0) return true;
            if (CheckDefaultValidate == false)
            {
/*
                string mess = "Warning : The server may have an invalid certificate" + "\n" + "Problem Description : "  + GetProblemMessage((CertificateProblem)problem) + "\n" + "Do you want to continue...";
                DialogResult dr = MessageBox.Show(mess,Application.ProductName,MessageBoxButtons.YesNo) ;
                if (dr == DialogResult.Yes)
                {
                    CheckDefaultValidate=true;
                }
*/
				CheckDefaultValidate=true;		// NO COMMENT, on fera un jour...
            }
				
            return CheckDefaultValidate; 
			
        }
    
        private String GetProblemMessage(CertificateProblem Problem)
        {
            String ProblemMessage = "";
            CertificateProblem problemList = new CertificateProblem();
            String ProblemCodeName = Enum.GetName(problemList.GetType(),Problem);
            if(ProblemCodeName != null)
                ProblemMessage = ProblemMessage + ProblemCodeName;
            else
                ProblemMessage = "Unknown Certificate Problem";
            return ProblemMessage;
        }
	
    }
	#endregion
}
