﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Microsoft.TeamFoundation.Client;
using Microsoft.TeamFoundation.VersionControl.Client;

namespace TFSHelpers
{
	public partial class BrowseSCItem : Form
	{
		public BrowseSCItem(VersionControlServer sc)
		{
			m_SC	= sc;

			InitializeComponent();
		}
		public String m_SelectedPath
		{
			get { return m_SelItemURL.Text; }
		}
		public VersionControlServer	m_SC;

		private void BrowseSCItem_Shown(object sender, EventArgs e)
		{
			// Create the root item
			TreeNode root = m_Tree.Nodes.Add("Root", "$", 0, 0);

			// Get all the projects
			TeamProject[] projs = m_SC.GetAllTeamProjects(false);

			foreach (TeamProject tp in projs)
			{
				TreeNode curtn = root.Nodes.Add("TeamProject", tp.Name, 1, 1);
				curtn.Nodes.Add("FakeNode", "FakeNode", 2, 2);
			}

			// On expand le Root
			root.Expand();
		}

		private void m_Tree_BeforeExpand(object sender, TreeViewCancelEventArgs e)
		{
			// On s'apprête à expander un item dont le contenu n'est pas encore chargé ?
			if ((e.Node.FirstNode!=null) && (e.Node.FirstNode.Name=="FakeNode"))
			{
				// On supprime le FakeNode, car on va charger le contenu
				e.Node.FirstNode.Remove();

				// On récupère le chemin du node dont on veut affiche le contenu (server-side)
				String		itemurl = e.Node.FullPath;

				// On évalue tous les dossiers qui sont enfants direct du node qu'on expand
				ItemSet		items	= m_SC.GetItems(itemurl, VersionSpec.Latest, RecursionType.OneLevel, DeletedState.NonDeleted, ItemType.Folder);

				// On rajoute un Tree Item pour chacun de ces dossiers
				for (int i=1; i<items.Items.Length; i++)
				{
					Item it = items.Items[i];
					int li = it.ServerItem.LastIndexOf("/");
					String itemname = it.ServerItem.Substring(li+1);
					TreeNode curtn = e.Node.Nodes.Add("Folder", itemname, 2, 2);
					curtn.Nodes.Add("FakeNode", "FakeNode", 2, 2);
				}
			}

			// Mise à jour de l'icon si c'est un dossier et qu'il n'est pas vide
			if ((e.Node.Name=="Folder") && (e.Node.Nodes.Count>0))
			{
				e.Node.ImageIndex = 3;
			}
		}

		private void m_Tree_BeforeCollapse(object sender, TreeViewCancelEventArgs e)
		{
			// Mise à jour de l'icon si c'est un dossier
			if (e.Node.Name == "Folder")
			{
				e.Node.ImageIndex = 2;
			}
		}

		private void m_Tree_AfterSelect(object sender, TreeViewEventArgs e)
		{
			m_SelItemURL.Text = e.Node.FullPath;
		}
	}
}