using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.TeamFoundation.Server;
using Microsoft.TeamFoundation.Client;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
using Microsoft.TeamFoundation.WorkItemTracking.Controls;
using WIManager;

namespace WICreator
{
	public partial class CheckDB : Form
	{
		MainForm							m_Owner				= null;
		TeamFoundationServer				m_TFS				= null;
		WorkItemStore						m_WIS				= null;
		Project								m_Project			= null;
		WINetworkModel						m_NetModel			= null;

		public CheckDB(MainForm owner, TeamFoundationServer tfs, Project proj)
		{
			InitializeComponent();

			this.Text	= String.Format("Checking Work Item database integrity of {0}", proj.Name);

			m_Owner		= owner;
			m_TFS		= tfs;
			m_WIS		= m_TFS.GetService(typeof(WorkItemStore)) as WorkItemStore;
			m_NetModel	= new WINetworkModel(m_WIS, null);
			m_Project	= proj;

		}

		void AddOutput(String str, bool alsostatus)
		{
			m_Output.Select(m_Output.TextLength, 0);
			m_Output.Paste(str);

			if (alsostatus)
			{
				SetStatusText(str);
			}
		}

		void SetStatusText(String txt)
		{
			m_StatusText.Text = txt;
			m_StatusStrip.Update();
		}

		private void m_Go_Click(object sender, EventArgs e)
		{
			bool fix = m_AutoFix.Checked;

			// EVALUATE ALL THE WORK ITEMS HAVING A PARENT
			Hashtable h = new Hashtable();
			h.Add("project", m_Project.Name);

			AddOutput("Query the Work Items supposed to be linked...", true);
			String strq = "SELECT System.ID from WorkItems where[System.TeamProject]=@project AND ([ParentWI]<>'' AND [ParentWI]<>'-1' AND [ParentWI]<>'0')";
			WorkItemCollection wicol = m_WIS.Query(strq, h);
			AddOutput(String.Format("{0} found.\r\n", wicol.Count), false);


			// CHECK FOR CHILDREN INTEGRITY
			// FOR EACH WORK ITEM CHECK IF IT'S PRESENT IN ITS PARENT

			AddOutput("Check for children integrity (cyclic links, will be fixed regardless of the option)\r\n", true);
			m_Progress.Minimum	= 1;
			m_Progress.Maximum	= wicol.Count;
			m_Progress.Value	= 1;

			foreach (WorkItem _wi in wicol)
			{
				// Get from the Network Model, to benefit of the cache
				WorkItem wi = m_NetModel.GetWorkItem(_wi.Id);
				SetStatusText(String.Format("Analyzing Work Item {0}...", wi.Id));

				List<int> children = new List<int>();
				WorkItem prev = null;
				WorkItem cur  = m_NetModel.GetFirstChildWorkItem(wi);
				while (cur != null)
				{
					if (children.Contains(cur.Id))
					{
						AddOutput(String.Format("Cyclic link in the Work Item [{0}], fixing...", wi.Id), false);
						m_NetModel.SetFieldNextWI(prev, 0);
						if (prev.IsValid())
						{
							prev.Save();
						}
						AddOutput("Done!\r\n", false);
						break;
					}

					children.Add(cur.Id);
					prev = cur;
					cur  = m_NetModel.GetNextWorkItem(cur);
				}

				m_Progress.Increment(1);
				Application.DoEvents();
			}

			AddOutput("Done\r\n\r\n", true);

			// FOR EACH WORK ITEM CHECK IF IT'S PRESENT IN ITS PARENT
			AddOutput("Detecting disguised orphans and multiple referenced ones...\r\n", true);
			m_Progress.Minimum	= 1;
			m_Progress.Maximum	= wicol.Count;
			m_Progress.Value	= 1;

			foreach (WorkItem _wi in wicol)
			{
				// Get from the Network Model, to benefit of the cache
				WorkItem wi = m_NetModel.GetWorkItem(_wi.Id);
				SetStatusText(String.Format("Analyzing Work Item {0}...", wi.Id));

				// Get its parent
				WorkItem wiparent = m_NetModel.WIGetParent(wi);
				
				// Parse the children to find wi.id
				int			found = 0;
				WorkItem	lastwi = null;
				foreach (WorkItem cur in m_NetModel.WIGetChildren(wiparent))
				{
					if (cur.Id == wi.Id)
					{
						found++;
					}
					lastwi = cur;
				}

				// Not found ? It's a disguised orphan
				if (found==0)
				{
					AddOutput(String.Format("Work Item [{0}] is not linked into its parent{1}", wi.Id, fix ? "..." : "\r\n"), true);
					if (fix)
					{
						if (lastwi == null)
						{
							m_NetModel.SetFieldFirstChildWI(wiparent, wi.Id);
							lastwi = wiparent;
						}
						else
						{
							m_NetModel.SetFieldNextWI(lastwi, wi.Id);
						}

						if (lastwi.IsValid())
						{
							lastwi.Save();

							m_NetModel.SetFieldNextWI(wi, 0);
							if (wi.IsDirty)
							{
								if (wi.IsValid())
								{
									wi.Save();
								}
								else
								{
									AddOutput(String.Format("Work Item [{0}] is invalid, can't save it, fix impossible!\r\n", wi.Id), false);
								}
							}

							AddOutput("Fixed!\r\n", false);
						}
						else
						{
							AddOutput(String.Format("Work Item [{0}] is invalid, can't save it, fix impossible!\r\n", lastwi.Id), false);
						}
					}
				}

				else if (found > 1)
				{
					AddOutput(String.Format("Work Item [{0}] is referenced multiple times in its parent!\r\n", wi.Id), true);
				}

				m_Progress.Increment(1);
				Application.DoEvents();
			}
			AddOutput("Done\r\n\r\n", true);


			// CHECK A GIVEN WORK ITEM HAS ITS CHILDREN REALLY OWNED BY IT
			AddOutput("Check children consistency...\r\n", true);
			m_Progress.Minimum	= 1;
			m_Progress.Maximum	= wicol.Count;
			m_Progress.Value	= 1;

			foreach (WorkItem _wi in wicol)
			{
				// Get from the Network Model, to benefit of the cache
				WorkItem wi = m_NetModel.GetWorkItem(_wi.Id);
				SetStatusText(String.Format("Analyzing Work Item {0}...", wi.Id));

				WorkItem prev = null;
				WorkItem cur  = m_NetModel.GetFirstChildWorkItem(wi);
				while (cur != null)
				{
					if (m_NetModel.GetFieldParentWI(cur) != wi.Id)
					{
						AddOutput(String.Format("Work Item [{0}] was got from [{1}] but has [{2}] as parent.", 
							cur.Id, wi.Id, m_NetModel.GetFieldParentWI(cur)), false);

						if (fix)
						{
							AddOutput(" Fixing...", false);

							WorkItem changedwi	= null;
							WorkItem nextwi		= m_NetModel.GetNextWorkItem(cur);
							while ((nextwi!=null) && (m_NetModel.GetFieldParentWI(nextwi) != wi.Id))
							{
								nextwi = m_NetModel.GetNextWorkItem(nextwi);
							}

							if (prev == null)
							{
								changedwi = wi;
								m_NetModel.SetFieldFirstChildWI(changedwi, (nextwi==null) ? 0 : nextwi.Id);
							}
							else
							{
								changedwi = prev;
								m_NetModel.SetFieldNextWI(changedwi, (nextwi==null) ? 0 : nextwi.Id);
							}

							if (changedwi.IsDirty)
							{
								if (changedwi.IsValid())
								{
									changedwi.Save();
								}
								else
								{
									AddOutput(String.Format("Work Item [{0}] is invalid, can't save it, fix impossible!\r\n", changedwi.Id), false);
								}
							}

							AddOutput(" Done.\r\n", false);
						
							prev = cur;
							cur  = nextwi;
						}
						else
						{
							AddOutput("\r\n", false);

							prev = cur;
							cur  = m_NetModel.GetNextWorkItem(cur);
						}
					}
					else
					{
						prev = cur;
						cur = m_NetModel.GetNextWorkItem(cur);
					}
				}

				m_Progress.Increment(1);
				Application.DoEvents();
			}
			AddOutput("Done\r\n\r\n", true);
		}

		private void CheckDB_Load(object sender, EventArgs e)
		{

		}
	}
}