using System;
using System.Collections;
using System.Collections.Generic;
using System.Drawing;
using System.ComponentModel;
using System.Windows.Forms;
using Microsoft.TeamFoundation.Server;
using Microsoft.TeamFoundation.Client;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
using Microsoft.TeamFoundation.VersionControl.Client;
using Microsoft.TeamFoundation.WorkItemTracking.Controls;
using TFSHelpers;
using WIManager;
using WICreator.Properties;

namespace WICreator
{
	public partial class MakeReleaseForm : Form
	{
		#region Action class implementation
		public class Action : IDisposable
		{
			public Action(MakeReleaseForm owner, String beginmsg, String endmsg)
			{
				// Set the owner
				m_Owner = owner;

				// Push the action in the owner's stack
				m_Owner.m_Actions.Push(this);

				// Set the end message
				m_EndMsg = endmsg;

				// Add the message
				owner.m_Output.Select(owner.m_Output.TextLength, 0);
				owner.m_Output.Paste(beginmsg);

				// Set the cursor to wait
				Cursor.Current = Cursors.WaitCursor;
			}

			public void Dispose()
			{
				// Pop from the stack
				m_Owner.m_Actions.Pop();
				if (m_EndMsg!=null)
				{
					// Add the message
					m_Owner.m_Output.Select(m_Owner.m_Output.TextLength, 0);
					m_Owner.m_Output.Paste(m_EndMsg);

				}
				Cursor.Current = Cursors.Default;					// Release the Wait Cursor by putting the default one
			}

			public	String		m_EndMsg;

			MakeReleaseForm		m_Owner;
		};
		#endregion
		Stack<Action>						m_Actions			= new Stack<Action>();

		MainForm							m_Owner				= null;
		TeamFoundationServer				m_TFS				= null;
		WorkItemStore						m_WIS				= null;
		VersionControlServer				m_SC				= null;
		WINetworkModel						m_NetModel			= null;

		SortedDictionary<int, WorkItem>		m_MergeableWIList	= null;
		SortedDictionary<int, WorkItem>		m_MergeList			= null;

		bool								m_UsedMergeCandidates = true;

		SortedDictionary<int, Changeset>	m_CandidatesDict	= new SortedDictionary<int, Changeset>();
		Changeset[]							m_CandidatesArray;
		bool								m_DoMergeCandidates	= true;
		Changeset[]							m_MergeCandidates
		{
			get
			{
				if (m_DoMergeCandidates == false)
				{
					return m_CandidatesArray;
				}

//				UpdateBuggedMergeCandidates();

				MergeCandidate[] mcs = m_SC.GetMergeCandidates(m_SrcBranch.Text, m_DstBranch.Text, RecursionType.Full);
				foreach (MergeCandidate mc in mcs)
				{
					try
					{
						m_CandidatesDict.Add(mc.Changeset.ChangesetId, mc.Changeset);
					}
					catch (Exception)
					{
					}
				}


				m_DoMergeCandidates = false;
				m_CandidatesArray = new Changeset[m_CandidatesDict.Values.Count];
				m_CandidatesDict.Values.CopyTo(m_CandidatesArray, 0);
				return m_CandidatesArray;
			}
		}

		TeamProject				m_TP
		{
			get
			{
				return m_SC.GetTeamProjectForServerPath(m_SrcBranch.Text);
			}
		}

		public MakeReleaseForm(MainForm owner, TeamFoundationServer tfs)
		{
			m_Owner		= owner;
			m_TFS		= tfs;
			m_SC		= m_TFS.GetService(typeof(VersionControlServer)) as VersionControlServer;
			m_WIS		= m_TFS.GetService(typeof(WorkItemStore)) as WorkItemStore;
			m_NetModel	= new WINetworkModel(m_WIS, null);

			InitializeComponent();
		}

		void AddOutput(String str)
		{
			m_Output.Select(m_Output.TextLength, 0);
			m_Output.Paste(str);
		}

		private void UpdateBuggedMergeCandidates()
		{
			// GetMergeCandidates doesn't return all the changesets it should, it forgets the changesets containing
			// new files, so let's do this by ourselves
			ProcessItemSet(m_SrcBranch.Text, m_SrcBranch.Text.Length);
		}

		private void ProcessItemSet(String path, int branchlength)
		{
			ItemSet itemset = m_SC.GetItems(path);
			
			foreach(Item it in itemset.Items)
			{
				String dstname = m_DstBranch.Text + it.ServerItem.Substring(branchlength);

				// The file/folder doesn't exist on the destination branch ? That would be our case !
				try 
				{	        
					m_SC.GetItem(dstname);
				}
				catch (Exception)
				{
					IEnumerable ie = m_SC.QueryHistory(it.ServerItem, VersionSpec.Latest, 0, RecursionType.Full, null, null, null, 10000, false, true);
					foreach (Changeset cs in ie)
					{
						try
						{
							m_CandidatesDict.Add(cs.ChangesetId, cs);
						}
						catch (Exception)
						{
						}
					}
					continue;
				}

				// The item exists on the destination and it's a folder: recurse!
				if (it.ItemType == ItemType.Folder)
				{
					ProcessItemSet(it.ServerItem + "/*", branchlength);
				}

			}

		}

		private void MakeReleaseForm_Load(object sender, EventArgs e)
		{
			// Initialize the workspace combobox with the Workspaces owned by the current user on the current computer
			Workspace[] wksps = m_SC.QueryWorkspaces(null, m_TFS.AuthenticatedUserName, Environment.MachineName);
			foreach (Workspace wksp in wksps)
			{
				m_WkspSel.Items.Add(wksp.Name);
			}

			// Select the last used Workspace (if any, or select the first one)
			m_WkspSel.SelectedText = Settings.Default.MR_Workspace;
			if (m_WkspSel.SelectedText == "")
			{
				m_WkspSel.SelectedIndex = 0;
			}

			// Set the Src/Dst branches
			m_SrcBranch.Text = Settings.Default.MR_SrcBranch;
			m_DstBranch.Text = Settings.Default.MR_DstBranch;

			// Set the iteration
			m_Iteration.Text = Settings.Default.MR_Iteration;

			// Set the Query
			String strq = String.Format(
@"SELECT System.ID from WorkItems where
[System.TeamProject]=@project AND
[System.State]='Resolved' AND 
[System.IterationPath] under @iteration");
			m_WIQuery.Text	 = (Settings.Default.MR_WIQuery=="") ? strq : Settings.Default.MR_WIQuery;

			// Set the Autoclose
			m_AutoCloseWI.Checked			= Settings.Default.MR_AutoClose;

			// Get the "Change Iteration for unmerged Work Item" settings
			m_ChangeIteration.Checked		= Settings.Default.MR_ChangeIteration;
			m_NewTargetIteration.Enabled	= m_ChangeIteration.Checked;
			m_SelectNewIterationBtn.Enabled	= m_ChangeIteration.Checked;

			// Get the new target iteration
			m_NewTargetIteration.Text		= Settings.Default.MR_NewTargetIteration;

			// Used the same Image list than the mainform
			m_WITree.ImageList				= m_Owner.TreeIL;
		}

		Workspace		m_Workspace
		{
			get
			{
				try
				{
					return m_SC.GetWorkspace(m_WkspSel.SelectedItem.ToString(), m_TFS.AuthenticatedUserName);
				}
				catch(Exception)
				{
					return null;
				}
			}
		}

		private void BrowseForSourceBranch(object sender, EventArgs e)
		{
			BrowseSCItem	form = new BrowseSCItem(m_SC);
			if (form.ShowDialog(this) != DialogResult.OK)
			{
				return;
			}

			m_SrcBranch.Text = form.m_SelectedPath;
			m_DoMergeCandidates = true;
		}

		private void BrowseForDstBranch(object sender, EventArgs e)
		{
			BrowseSCItem	form = new BrowseSCItem(m_SC);
			if (form.ShowDialog(this) != DialogResult.OK)
			{
				return;
			}

			m_DstBranch.Text = form.m_SelectedPath;
			m_DoMergeCandidates = true;
		}

		private void SelectIteration(object sender, EventArgs e)
		{
			try
			{
				BrowseAreaIteration form = new BrowseAreaIteration(m_TFS, m_TP, false);
				if (form.ShowDialog(this) != DialogResult.OK)
				{
					return;
				}

				m_Iteration.Text = form.m_SelItem.Text;
			}
			catch (Exception)
			{
				return;
			}
		}
		
		private void ExecWIQuery(object sender, EventArgs e)
		{
			try
			{
				// We associate the @iteration macro to the current iteration
				Hashtable h = new Hashtable();
				h.Add("iteration", m_Iteration.Text);
				h.Add("project", m_TP.Name);

				// Get all the merge candidates: the changesets that are present in the src branch but not in the dst one
				List<int> csfrommc = new List<int>();
				if (m_UsedMergeCandidates)
				{
					using (new Action(this, "Get merge candidates, can take a while...", "Done\r\n"))
					{
						foreach (Changeset cs in m_MergeCandidates)
						{
							csfrommc.Add(cs.ChangesetId);
						}
					}
				}
				else
				{
					IEnumerable ie = m_SC.QueryHistory(m_SrcBranch.Text, VersionSpec.Latest, 0, RecursionType.Full, null, null, null, 10000, false, true);
					foreach (Changeset cs in ie)
					{
						csfrommc.Add(cs.ChangesetId);
					}
				}

				// Make the Work Item Query
				WorkItemCollection wicol = null;
				using (new Action(this, "Making the Work Item Query...", "Done\r\n"))
				{
					wicol = m_WIS.Query(m_WIQuery.Text, h);
				}

				// Build a dictionary of the CS belonging to each returned WI
				m_MergeableWIList = new SortedDictionary<int,WorkItem>();

				// Compute the list of merge-able Work Item
				using (new Action(this, "Computing the list of merge-able Work Items...\r\n", "Done\r\n"))
				{
					// Build a dictionary of the CS belonging to each returned WI
					SortedDictionary<int, WorkItem>	cswilist = new SortedDictionary<int,WorkItem>();

					// For each Work Item returned by the query, evaluate all the associated changesets, and add them
					try
					{
						foreach (WorkItem curwi in wicol)
						{
							// Evaluate the Links of the Work Item
							foreach (Link link in curwi.Links)
							{
								// Get the current link, cast to an external link
								ExternalLink el = link as ExternalLink;
								if ((el != null) && (el.ArtifactLinkType.Name == "Fixed in Changeset"))
								{
									int lasti = el.LinkedArtifactUri.LastIndexOf('/');
									int csid = int.Parse(el.LinkedArtifactUri.Substring(lasti+1));
									try
									{
										cswilist.Add(csid, curwi);
									}
									catch (Exception)
									{
										WorkItem prev = cswilist[csid];
										AddOutput(String.Format("WARNING: Work Item {0} is ignored because the "+
																"changeset {1} is already referenced in the Work Item {2}\r\n",
										curwi.Id, csid, prev.Id));
									}
								}
							}
						}

					}
					catch (Exception ex)
					{
						ex.ToString();
					}
					// Filter the dictionary by removing the entries referencing changeset not returned by the
					// GetMergeCandidates
					foreach (int curcs in csfrommc)
					{
						WorkItem wi;

						// Check if the CS is in the dictionary
						if (cswilist.TryGetValue(curcs, out wi))
						{
							m_MergeableWIList.Add(curcs, wi);
						}
					}
				}

				using (new Action(this, "Building Tree... 0%", "Done\r\n"))
				{
					m_WITree.Nodes.Clear();
					m_WITree.BeginUpdate();

					// We add the work item in the tree
					int i		= 0;
					int count	= m_MergeableWIList.Count;
					foreach (WorkItem wi in m_MergeableWIList.Values)
					{
						m_Output.Select(m_Output.Text.Length-3, 3);
						m_Output.Paste(String.Format("{0,2}%", i*100/count));

						AddWIToTree(wi.Id);
						i++;
					}
					m_Output.Select(m_Output.Text.Length-3, 3);
					m_Output.Paste("");

					// Reactivate the rendering of the control
					m_WITree.EndUpdate();
				}
			}
			catch (Exception)
			{
			}

			UpdateTreeCheckState(m_WITree.Nodes);
		}

		private bool UpdateTreeCheckState(TreeNodeCollection nodecol)
		{
			bool nchecked = true;
			foreach (TreeNode tn in nodecol)
			{
				tn.Checked = UpdateTreeCheckState(tn.Nodes);
				nchecked  &= tn.Checked;
			}
			return nchecked;
		}

		private void AddWIToTree(int wiid)
		{
			// Get the Work Item
			WorkItem wi = m_NetModel.GetWorkItem(wiid);

			// Nothing ???
			if (wi == null)
			{
				return;
			}

			// Start from the given Work Item, find the Root, and store the whole path between these two
			List<WorkItem> wilist = new List<WorkItem>();
			
			WorkItem	curwi = wi;
			while (curwi != null)
			{
				wilist.Insert(0, curwi);				// As we go up in the hierarchy, we insert always at the first position
				curwi = m_NetModel.WIGetParent(curwi);	// to keep the right direction after
			}

			TreeNode			tn	= null;
			TreeNodeCollection	tnc = m_WITree.Nodes;
			foreach (WorkItem _curwi in wilist)
			{
				tn  = GetTreeNode(tnc, _curwi, true);
				tnc = tn.Nodes;
			}
			
			if (tn != null)
			{
				tn.ForeColor = Color.Black;
				tn.Checked	 = true;
			}
		}

		private TreeNode GetTreeNode(TreeNodeCollection owner, WorkItem wi, bool create)
		{
			foreach (TreeNode curtn in owner)
			{
				WorkItem curwi = curtn.Tag as WorkItem;
				if (curwi==null)
				{
					continue;
				}

				if (curwi.Id == wi.Id)
				{
					return curtn;
				}
			}

			if (create == false)
			{
				return null;
			}

			TreeNode tn = CreateTreeNode(wi);
			if (tn != null)
			{
				owner.Add(tn);
			}
			return tn;
		}

		private TreeNode CreateTreeNode(WorkItem wi)
		{
			String name = String.Format("[{0}] {1}-{2}", wi.Id.ToString(), wi.Title, wi.State);

			// Create the node
			TreeNode	tn	= new TreeNode(name);
			tn.Tag			= wi;

			// Picture index from the Work Item's type
			int ii = m_WITree.ImageList.Images.IndexOfKey(wi.Type.Name);
			if (ii==-1)
			{
				ii = m_WITree.ImageList.Images.IndexOfKey("Default");
			}
			tn.ImageIndex = tn.SelectedImageIndex = ii;
			tn.ForeColor = Color.Gray;

			return tn;
		}

		private bool HasFailed(GetStatus st)
		{
			// Failures ?
			if (st.NumFailures > 0)
			{
				String failuretext = "";
				Failure[] failures = st.GetFailures();
				bool error = false;
				foreach (Failure f in failures)
				{
					failuretext += "\t" + f.ToString() + "\r\n";
					if (f.Severity == SeverityType.Error)
					{
						error = true;
					}
				}

				if (error == false)
				{
					return false;
				}
				AddOutput(String.Format("\r\nFailure during the merge:\r\n{0}", failuretext));

				return true;
			}

			return false;
		}

		private void MakeRelease(object sender, EventArgs e)
		{
			try
			{
				///////////////////////////////////////////////////////////////////////////////////////////////////////////
				// Check if the workspace is not empty
				if (m_Workspace.GetPendingChanges().Length>0)
				{
					MessageBox.Show("The selected Workspace is not empty, you must clean it up before making a release!", 
						"Work Item Creator", MessageBoxButtons.OK);
					return;
				}

				int lasti = m_DstBranch.Text.LastIndexOf('/');
				String targetrel = m_DstBranch.Text.Substring(lasti+1);
				
				DialogResult dr = MessageBox.Show("Do you want to make a get of the branches before making the release ?", "Work Item Creator", 
					MessageBoxButtons.YesNoCancel, MessageBoxIcon.Information);

				if (dr == DialogResult.Cancel)
				{
					return;
				}

				///////////////////////////////////////////////////////////////////////////////////////////////////////////
				// Get the latest version of the sources for both source and destination branches
				if (dr == DialogResult.Yes)
				{
					AddOutput(String.Format("Get files (forced) of branches {0} and {1}, can take a while...\r\n", m_SrcBranch.Text, m_DstBranch.Text));

					// Make the get
					String[] branches = new string[]{m_SrcBranch.Text, m_DstBranch.Text};
					GetStatus gs = m_Workspace.Get(branches, VersionSpec.Latest, RecursionType.Full, GetOptions.GetAll);

					AddOutput("Done!\r\n");

					// Conflicts ?
					if (gs.NoActionNeeded == false)
					{
						if (gs.NumFailures>0)
						{
							String strf = "";
							foreach (Failure f in gs.GetFailures())
							{
								strf += f.Message + "\r\n";
							}
							MessageBox.Show(String.Format("There were {0} failures, can't go further...\r\n{1}", gs.NumFailures, strf), "Work Item Creator", MessageBoxButtons.OK, MessageBoxIcon.Error);
							return;
						}

						if (gs.NumConflicts>0)
						{
							AddOutput(String.Format("\r\nThere are {0} conflicts for this merge, manual action must be done...\r\n", gs.NumConflicts));
							Conflict[] conflicts = m_Workspace.QueryConflicts(new string[] { m_DstBranch.Text }, true);
							ResolveConflictsForm form = new ResolveConflictsForm(ref conflicts, m_Workspace);
							if (form.ShowDialog() != DialogResult.OK)
							{
								AddOutput("\r\nConflict not resolved, operation aborted!");
								return;
							}
						}
					}
				}

				///////////////////////////////////////////////////////////////////////////////////////////////////////////
				// Create a dictionary that will contain the changesets to merge
				m_MergeList = new SortedDictionary<int,WorkItem>(m_MergeableWIList);

				// Remove the changesets corresponding to non selected work items
				using (new Action(this, "Computing changesets of selected Work Items...", "Done\r\n"))
				{
					FilterChangesets(m_WITree.Nodes, m_MergeList);
				}

				// Display a dialog that sums up all the changeset being merged
				ChangesetsToMerge	csform = new ChangesetsToMerge(m_SC, ref m_MergeList);
				if (csform.ShowDialog() != DialogResult.OK)
				{
					return;
				}

				///////////////////////////////////////////////////////////////////////////////////////////////////////////
				// Now evaluate the remaining list of changesets
				foreach (KeyValuePair<int, WorkItem> kvp in m_MergeList)
				{
					////////////////////////////////////////////////////////////////////////////////////////////////////
					// Check if the workspace is not empty. There's no reason for such case to happen, but it looks like
					// it happens sometime, so it's foremost for debugging purpose
					if (m_Workspace.GetPendingChanges().Length > 0)
					{
						if (MessageBox.Show("For some reason I don't fully understand, your workspace is not empty, and that is an unexpected behavior.\r\nDo you want to stop the process ?",
							"Work Item Creator", MessageBoxButtons.YesNo, MessageBoxIcon.Information) == DialogResult.Yes)
						{
							return;
						}
					}

					// Find the ChangeSet number
					int csid = kvp.Key;

					// Do the Merge
					AddOutput(String.Format("Merge the changeset {0}...", csid));
					ChangesetVersionSpec csvs = new ChangesetVersionSpec(csid);
					GetStatus st = null;
					try
					{
						st = m_Workspace.Merge(m_SrcBranch.Text, m_DstBranch.Text, csvs, csvs, LockLevel.None, RecursionType.Full, MergeOptions.None);
					}
					catch (Exception ex)
					{
						AddOutput(ex.ToString());
					}
					// Merge failed ?
					if (HasFailed(st))
					{
						AddOutput("Process next changeset...\r\n");
						continue;
					}

					// Nothing has been merge, weird...continue to the next changeset
					else  if ((st.NumOperations==0) && (st.NumConflicts==0))
					{
						AddOutput("Nothing to merge on this changeset...\r\n");
					}

					// Check if we have some additional actions to perform
					else if (st.NoActionNeeded == false)
					{
						// There's some conflicts
						if (st.NumConflicts>0)
						{
							AddOutput(String.Format("\r\nThere are {0} conflicts for this merge, manual action must be done...\r\n", st.NumConflicts));
							Conflict[] conflicts = m_Workspace.QueryConflicts(new string[] { m_DstBranch.Text }, true);
							ResolveConflictsForm form = new ResolveConflictsForm(ref conflicts, m_Workspace);
							if (form.ShowDialog() != DialogResult.OK)
							{
								AddOutput("Conflict resolution failed, the merge is aborted, but the operation continues!\r\n");
								m_Workspace.Undo(new string[]{m_SrcBranch.Text}, RecursionType.Full, true);
								continue;
							}
						}

						// Proceed the checking
						WorkItemCheckinInfo[] wicilist = new WorkItemCheckinInfo[1];
						Changeset cs	= m_SC.GetChangeset(csid);
						String srcb		= m_SrcBranch.Text.Substring(m_SrcBranch.Text.LastIndexOf('/')+1);
						String dstb		= m_DstBranch.Text.Substring(m_DstBranch.Text.LastIndexOf('/')+1);
						String comments = String.Format("Report of CS {0} from {1} to {2}\r\n{3}", csid, srcb, dstb, cs.Comment);

						// Evaluate the list of the changesets we merged
						wicilist[0] = new WorkItemCheckinInfo(kvp.Value, WorkItemCheckinAction.Associate);
						
						// Build the Checkin Note object
						CheckinNoteFieldValue[] cinfv = new CheckinNoteFieldValue[1];
						cinfv[0] = new CheckinNoteFieldValue("Code Reviewer", "");
						CheckinNote ci = new CheckinNote(cinfv);

						// EVALUATE IF WE CAN CHECKIN
						CheckinEvaluationResult cer = m_Workspace.EvaluateCheckin(CheckinEvaluationOptions.Conflicts, m_Workspace.GetPendingChanges(), 
							m_Workspace.GetPendingChanges(), comments, ci, wicilist);

						// CONFLICTS ? RESOLVE THEM
						if (cer.Conflicts.Length > 0)
						{
							AddOutput(String.Format("\r\nThere are {0} conflicts for this merge, manual action must be done...\r\n", cer.Conflicts.Length));
							Conflict[] conflicts = m_Workspace.QueryConflicts(new string[] { m_DstBranch.Text }, true);
							ResolveConflictsForm form = new ResolveConflictsForm(ref conflicts, m_Workspace);
							if (form.ShowDialog() != DialogResult.OK)
							{
								AddOutput("Conflict resolution failed, the merge is aborted, but the operation continues!\r\n");
								m_Workspace.Undo(new string[] { m_SrcBranch.Text }, RecursionType.Full, true);
								continue;
							}
						}

						// DO THE CHECKIN
						try
						{
							int newcsid = m_Workspace.CheckIn(m_Workspace.GetPendingChanges(), comments, null, wicilist, null);
							AddOutput(String.Format("Changeset {0} created.\r\n", newcsid));

						}
						catch (Exception ex)
						{
							MessageBox.Show(String.Format("The check-in failed: {0}", ex.ToString()), 
								"Work Item Creator", MessageBoxButtons.OK, MessageBoxIcon.Error);
						}
					}
				}

				// Need to create a label ?
				if ((m_CreateLabelCheck.Checked==true) && (m_NewLabelName.Text.Length>0))
				{
					using (new Action(this, String.Format("\r\nCreate the label {0} for {1}", m_NewLabelName.Text, m_DstBranch.Text), "Done\r\n"))
					{
						VersionControlLabel label = new VersionControlLabel(m_SC, m_NewLabelName.Text, m_TFS.AuthenticatedUserName,
							m_DstBranch.Text, "Label Created with the WICreator");
						LabelItemSpec[] lis = new LabelItemSpec[1];
						ItemSpec its = new ItemSpec(m_DstBranch.Text, RecursionType.Full);
						lis[0] = new LabelItemSpec(its, VersionSpec.Latest, false);
						m_SC.CreateLabel(label, lis, LabelChildOption.Replace);
					}
				}

				// We change the iteration of un-merged Work Items ?
				if ((m_ChangeIteration.Checked == true) && (m_NewTargetIteration.Text.Length>0))
				{
					using (new Action(this, "\r\nUpdate iteration of unmerged Work Items", "Done\r\n"))
					{
						UpdateWIsIteration(m_WITree.Nodes, m_NewTargetIteration.Text);
					}
				}

				// Invalidate the cache of merge candidates as we've done a check-in
				m_DoMergeCandidates = true;
			}
			catch (Exception ex)
			{
				MessageBox.Show(String.Format("An unexpected error occurred during the Release process: {0}", ex.ToString()), 
					"Work Item Creator", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		void UpdateWIsIteration(TreeNodeCollection tnc, String newiteration)
		{
			foreach(TreeNode tn in tnc)
			{
				if (tn.Checked == false)
				{
					WorkItem wi = tn.Tag as WorkItem;
					if (wi != null)
					{
						wi.Open();
						wi.IterationPath = newiteration;
						wi.History = String.Format("Iteration requalified by the release process of WICreator.\r\nNew Iteration {0}", newiteration);
						wi.Save();
					}
				}
				UpdateWIsIteration(tn.Nodes, newiteration);
			}
		}

		void FilterChangesets(TreeNodeCollection tnc, SortedDictionary<int, WorkItem> mergelist)
		{
			foreach(TreeNode tn in tnc)
			{
				// The TreeNode is unchecked ?
				if (tn.Checked == false)
				{
					// Get the Work Item
					WorkItem wi = tn.Tag as WorkItem;
					if (wi!=null)
					{
						// Get all the changesets
						foreach (Link link in wi.Links)
						{
							ExternalLink el = link as ExternalLink;
							if ((el!=null) && (el.ArtifactLinkType.Name == "Fixed in Changeset"))
							{
								int lasti = el.LinkedArtifactUri.LastIndexOf('/');
								int csid = int.Parse(el.LinkedArtifactUri.Substring(lasti+1));
								try
								{
									mergelist.Remove(csid);
								}
								catch (Exception)
								{
								}
							}
						}
					}
				}

				// Recurse
				FilterChangesets(tn.Nodes, mergelist);
			}
		}

		private void WorkspaceSelChanged(object sender, EventArgs e)
		{
		}

		private void MakeReleaseForm_Closed(object sender, FormClosedEventArgs e)
		{
			// Save the settings
			Settings.Default.MR_Workspace			= m_WkspSel.SelectedText;
			Settings.Default.MR_SrcBranch			= m_SrcBranch.Text;
			Settings.Default.MR_DstBranch			= m_DstBranch.Text;
			Settings.Default.MR_Iteration			= m_Iteration.Text;
			Settings.Default.MR_WIQuery				= m_WIQuery.Text;
			Settings.Default.MR_AutoClose			= m_AutoCloseWI.Checked;
			Settings.Default.MR_ChangeIteration		= m_ChangeIteration.Checked;
			Settings.Default.MR_NewTargetIteration	= m_NewTargetIteration.Text;


			Settings.Default.Save();
		}

		private void SelectedTreeItemChanged(object sender, TreeViewEventArgs e)
		{
			WorkItem wi = e.Node.Tag as WorkItem;
			if (wi!=null)
			{
				m_WIForm.Item = wi;
			}
		}

		private void SelectNewTargetIteration(object sender, EventArgs e)
		{
			try
			{
				BrowseAreaIteration form = new BrowseAreaIteration(m_TFS, m_TP, false);
				if (form.ShowDialog(this) != DialogResult.OK)
				{
					return;
				}

				m_NewTargetIteration.Text = form.m_SelItem.Text;
			}
			catch (Exception)
			{
				return;
			}
		}

		private void ChangeIterationCheckChanged(object sender, EventArgs e)
		{
			m_NewTargetIteration.Enabled	= m_ChangeIteration.Checked;
			m_SelectNewIterationBtn.Enabled	= m_ChangeIteration.Checked;
			m_NewTargetIteration.Update();
		}

		private void m_FindOrphanCSsBtn_Click(object sender, EventArgs e)
		{
			if (m_UsedMergeCandidates==false)
			{
				IEnumerable ie = m_SC.QueryHistory(m_SrcBranch.Text, VersionSpec.Latest, 0, RecursionType.Full, null, null, null, 10000, false, true);
				String fcs = ("Orphan Changesets: ");
				foreach (Changeset cs in ie)
				{
					if (cs.WorkItems.Length == 0)
					{
						fcs += String.Format("\t {0}", cs.ChangesetId);
					}
				}
				AddOutput(fcs);
			}
			else
			{
				// Parse the list of Merge Candidate, evaluate if there's some without a associated Work Item
				using (new Action(this, "Compute Merge Candidates", "\r\n"))
				{
					if (m_MergeCandidates == null)
					{
						AddOutput("Please select a source and destination branch before calling this function");
						return;
					}
				}
				// Parse the list of Merge Candidate, evaluate if there's some without a associated Work Item
				using (new Action(this, "Finding orphan changesets... 0%", "Done\r\n"))
				{
					String fcs = ("Orphan Changesets: ");
					int i = 0;
					int count = m_MergeCandidates.Length;
					foreach (Changeset cs in m_MergeCandidates)
					{
						m_Output.Select(m_Output.Text.Length - 3, 3);
						m_Output.Paste(String.Format("{0,2}%", i * 100 / count));

						if (cs.WorkItems.Length == 0)
						{
							fcs += String.Format("\t {0}", cs.ChangesetId);
						}
						i++;
					}
					AddOutput(fcs);
				}
			}
		}

		private void m_MergeCandidatesBtn_Click(object sender, EventArgs e)
		{
			foreach (Changeset cs in m_MergeCandidates)
			{
				AddOutput(String.Format("\r\nChange Set : {0}", cs.ChangesetId));
			}
		}

		private void ApplyCheckToChildren(TreeNode tn, bool itc)
		{
			tn.Checked = itc;

			foreach(TreeNode ctn in tn.Nodes)
			{
				ApplyCheckToChildren(ctn, itc);
			}
		}

		private void m_WITree_NodeMouseDoubleClick(object sender, TreeNodeMouseClickEventArgs e)
		{
			bool c = e.Node.Checked;
			e.Node.Toggle();
			ApplyCheckToChildren(e.Node, c);
			
		}

		private void m_PromoteBtn_Click(object sender, EventArgs e)
		{
			if (MessageBox.Show(String.Format("All merges will be made from {0} to {1}, do it ?", 
				m_SrcBranch.Text, m_DstBranch.Text), "WI Creator", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
			{
				return;
			}

			SortedDictionary<int, WorkItem>	list = new SortedDictionary<int,WorkItem>();
			foreach (Changeset cs in m_MergeCandidates)
			{
				list.Add(cs.ChangesetId, (cs.WorkItems.Length>0) ? cs.WorkItems[0] : null);
			}

			if (list.Count == 0)
			{
				MessageBox.Show("There are no merge candidates, nothing to merge then!");
				return;
			}

			ChangesetsToMerge dlg = new ChangesetsToMerge(m_SC, ref list);
			if (dlg.ShowDialog() != DialogResult.OK)
			{
				return;
			}

			if (list.Count == 0)
			{
				MessageBox.Show("There are no merge candidates, nothing to merge then!");
				return;
			}

			///////////////////////////////////////////////////////////////////////////////////////////////////////////
			// Now evaluate the remaining list of changesets
			foreach (KeyValuePair<int, WorkItem> kvp in list)
			{
				////////////////////////////////////////////////////////////////////////////////////////////////////
				// Check if the workspace is not empty. There's no reason for such case to happen, but it looks like
				// it happens sometime, so it's foremost for debugging purpose
				if (m_Workspace.GetPendingChanges().Length > 0)
				{
					if (MessageBox.Show("For some reason I don't fully understand, your workspace is not empty, and that is an unexpected behavior.\r\nDo you want to stop the process ?",
						"Work Item Creator", MessageBoxButtons.YesNo, MessageBoxIcon.Information) == DialogResult.Yes)
					{
						return;
					}
				}

				// Find the ChangeSet number
				int csid = kvp.Key;
				Changeset cs = m_SC.GetChangeset(csid);

				// Do the Merge
				AddOutput(String.Format("Merge the changeset {0}...", csid));
				ChangesetVersionSpec csvs = new ChangesetVersionSpec(csid);
				GetStatus st = null;
				try
				{
					st = m_Workspace.Merge(m_SrcBranch.Text, m_DstBranch.Text, csvs, csvs, LockLevel.None, RecursionType.Full, MergeOptions.ForceMerge);
				}
				catch (Exception ex)
				{
					AddOutput(ex.ToString());
				}

				// Merge failed ?
				if (HasFailed(st))
				{
					AddOutput("Process next changeset...\r\n");
					continue;
				}

				// Nothing has been merge, weird...continue to the next changeset
				else if ((st.NumOperations==0) && (st.NumConflicts==0))
				{
					AddOutput("Nothing to merge on this changeset...\r\n");
				}

				// Check if we have some additional actions to perform
				else if (st.NoActionNeeded == false)
				{
					// There's some conflicts
					if (st.NumConflicts>0)
					{
						AddOutput(String.Format("\r\nThere are {0} conflicts for this merge, manual action must be done...\r\n", st.NumConflicts));
						Conflict[] conflicts = m_Workspace.QueryConflicts(new string[] { m_DstBranch.Text }, true);
						ResolveConflictsForm form = new ResolveConflictsForm(ref conflicts, m_Workspace);
						if (form.ShowDialog() != DialogResult.OK)
						{
							AddOutput("Conflict resolution failed, the merge is aborted, but the operation continues!\r\n");
							m_Workspace.Undo(new string[] { m_SrcBranch.Text }, RecursionType.Full, true);
							continue;
						}
					}

					// Proceed the checking
					WorkItemCheckinInfo[] wicilist = new WorkItemCheckinInfo[cs.WorkItems.Length];
					String srcb		= m_SrcBranch.Text.Substring(m_SrcBranch.Text.LastIndexOf('/')+1);
					String dstb		= m_DstBranch.Text.Substring(m_DstBranch.Text.LastIndexOf('/')+1);
					String comments = String.Format("Report of CS {0} from {1} to {2}\r\n{3}", csid, srcb, dstb, cs.Comment);

					// Evaluate the list of the changesets we merged
					int i = 0;
					foreach (WorkItem curwi in cs.WorkItems)
					{
						wicilist[i++] = new WorkItemCheckinInfo(curwi, WorkItemCheckinAction.Associate);
					}
					
					// Build the Checkin Note object
					CheckinNoteFieldValue[] cinfv = new CheckinNoteFieldValue[1];
					cinfv[0] = new CheckinNoteFieldValue("Code Reviewer", "");
					CheckinNote ci = new CheckinNote(cinfv);

					// EVALUATE IF WE CAN CHECKIN
					CheckinEvaluationResult cer = m_Workspace.EvaluateCheckin(CheckinEvaluationOptions.Conflicts, m_Workspace.GetPendingChanges(), 
						m_Workspace.GetPendingChanges(), comments, ci, wicilist);

					// CONFLICTS ? RESOLVE THEM
					if (cer.Conflicts.Length > 0)
					{
						AddOutput(String.Format("\r\nThere are {0} conflicts for this merge, manual action must be done...\r\n", cer.Conflicts.Length));
						Conflict[] conflicts = m_Workspace.QueryConflicts(new string[] { m_DstBranch.Text }, true);
						ResolveConflictsForm form = new ResolveConflictsForm(ref conflicts, m_Workspace);
						if (form.ShowDialog() != DialogResult.OK)
						{
							AddOutput("Conflict resolution failed, the merge is aborted, but the operation continues!\r\n");
							m_Workspace.Undo(new string[] { m_SrcBranch.Text }, RecursionType.Full, true);
							continue;
						}
					}

					// DO THE CHECKIN
					int newcsid = m_Workspace.CheckIn(m_Workspace.GetPendingChanges(), comments, null, wicilist, null);
					AddOutput(String.Format("Changeset {0} created.\r\n", newcsid));
				}
			}

			// Invalidate the cache of merge candidates as we've done a check-in
			m_DoMergeCandidates = true;
		}
	}
}