using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.IO;
using System.Windows.Forms;
using Microsoft.TeamFoundation.Server;
using Microsoft.TeamFoundation.Client;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
using Microsoft.TeamFoundation.VersionControl.Client;
using Microsoft.TeamFoundation.WorkItemTracking.Controls;

namespace WICreator
{
	public partial class ResolveConflictsForm : Form
	{
		Conflict[]	m_Conflicts;
		int			m_Resolved;
		int			m_Remaining;
		Workspace	m_Workspace;

		public ResolveConflictsForm(ref Conflict[] conflicts, Workspace wksp)
		{
			InitializeComponent();

			m_Conflicts = conflicts;
			m_Workspace		= wksp;
		}

		private void ResolveConflictsForm_Load(object sender, EventArgs e)
		{
			ResolutionModes.SelectedIndex = 0;

			m_Resolved	= 0;
			m_Remaining	= m_Conflicts.Length;
			UpdateStatus();

			ConflictList.BeginUpdate();
			foreach (Conflict conflict in m_Conflicts)
			{
				ListViewItem lvi = new ListViewItem(
					new string[]
					{
						conflict.TheirServerItem.Substring(conflict.TheirServerItem.LastIndexOf('/')+1),
						conflict.BaseChangeType.ToString(),
						conflict.GetBriefMessage(),
						conflict.TheirServerItem.Substring(0, conflict.TheirServerItem.LastIndexOf('/')),
						conflict.YourServerItem.Substring(0, conflict.YourServerItem.LastIndexOf('/'))
					}
				);
				lvi.Tag = conflict;

				ConflictList.Items.Add(lvi);
			}
			ConflictList.EndUpdate();
		}

		private void UpdateStatus()
		{
			Status.Text = String.Format("{0} remaining, {1} resolved", m_Remaining, m_Resolved);
			Status.Invalidate();
			Status.Update();

			if (m_Remaining == 0)
			{
				ActionBtn.Text = "OK";
				ActionBtn.DialogResult = DialogResult.OK;
			}
		}

		private void ActionBtn_Click(object sender, EventArgs e)
		{
			if (ActionBtn.DialogResult == DialogResult.OK)
			{
				return;
			}

			foreach(ListViewItem lvi in ConflictList.Items)
			{
				if (lvi.Checked)
				{
					Conflict conflict = lvi.Tag as Conflict;

					// Merge
					if (ResolutionModes.SelectedIndex == 0)
					{
						// Do the merge
						if (m_Workspace.MergeContent(conflict, true) == false)
						{
							// Merge cancelled, do nothing, loop to the next one
							continue;
						}

						// Resolve the conflict
						conflict.Resolution = Resolution.AcceptMerge;
						m_Workspace.ResolveConflict(conflict);
					}

					// Take local
					else if (ResolutionModes.SelectedIndex == 1)
					{
						conflict.Resolution = Resolution.AcceptTheirs;
						m_Workspace.ResolveConflict(conflict);
					}

					// Take local
					else if (ResolutionModes.SelectedIndex == 2)
					{
						conflict.Resolution = Resolution.AcceptYours;
						m_Workspace.ResolveConflict(conflict);
					}

					// Update status, remove the item as it's processed
					m_Resolved ++;
					m_Remaining--;
					
					ConflictList.Items.Remove(lvi);
				}

				UpdateStatus();
			}
		}
	}
}