﻿using System;
using System.Net;
using System.Diagnostics;
using System.Threading;
using System.Web;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.Xml;
using System.Xml.Serialization;
using WIManager;

namespace WINetwork
{
	/// <summary>
	/// This service listen to the Checkin-events from TFS and runs the build
	/// after it matches Team Project name from the Web.Config file. 
	/// </summary>
	[WebService(Namespace = "http://schemas.microsoft.com/TeamFoundation/2005/06/Services/Notification/03")]
	[WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
	public class Service : System.Web.Services.WebService
	{
		private		Object	HierarchyWorkerSync = new object();

		[SoapDocumentMethod("http://schemas.microsoft.com/TeamFoundation/2005/06/Services/Notification/03/Notify", RequestNamespace = "http://schemas.microsoft.com/TeamFoundation/2005/06/Services/Notification/03")]
		[WebMethod]
		public void Notify(string eventXml)     // Do not change the names of these parameters
		{
			String		xmodelunc	= this.Server.MapPath(@"~/NetworkModels.xml");
			XmlDocument docmodel	= null;

			// Load the configuration file
			try
			{
				docmodel = new XmlDocument();
				docmodel.Load(xmodelunc);
			}
			catch (Exception)
			{
				return;
			}
			
			// Create a new handler for that request, and process it...
			WINotifyHandler handler = new WINotifyHandler(docmodel);

			// First check if we can process the Hierarchy Workers (if one is already running in a concurrent request we avoid it)
			bool processworkers = false;
			lock (HierarchyWorkerSync)
			{
				if (Application["HierarchyWorkerFlag"] == null)
				{
					Application["HierarchyWorkerFlag"] = 0;
				}

				if ((int)Application["HierarchyWorkerFlag"] == 0)
				{
					Application["HierarchyWorkerFlag"] = 1;
					processworkers = true;
				}
			}

			if (processworkers == true)
			{
				// Load the XML file that contains the timestamps
				String xmltsunc = Server.MapPath(@"~/HWTimeStamp.xml");
				XmlDocument doctimestamp = new XmlDocument();

				// Load the configuration file
				try
				{
					doctimestamp.Load(xmltsunc);
				}
				// If an exception is thrown, the file is not on the drive so initialize it
				catch (Exception)
				{
					doctimestamp.LoadXml("<?xml version=\"1.0\" encoding=\"utf-8\" ?><TimeStamps/>");
				}

				// Handle the Hierarchy Workers
				handler.ProcessHierarchyWorkers(eventXml, doctimestamp, xmltsunc);

				// Set the flag back to zero to indicate we finished the process
				lock (HierarchyWorkerSync)
				{
					Thread.Sleep(500);
					Application["HierarchyWorkerFlag"] = 0;
				}
			}

			// Do the WINetwork job
			handler.ProcessWIChangedAlert(eventXml);
		}
	}
}
